from load_data import LoadData
from transform import Transform
from train_test_split import TrainTestSplit
from train import NeuralNetwork
from training import training_NN 
from evaluation import train_evaluation, validation_evaluation
# from classifier import Classifier

class Classifier:
    def __init__(self):
        print("\nWelcome to the Iris Classifier.".capitalize())
        print("_"*50,'\n')
        
    def load_data(self, filename = None):
        dataset = LoadData(filename)
        dataframe = dataset.load_csv()
        
        '''Do the transformed of target class'''
        
        encoding = Transform(dataframe)
        encoded_dataset = encoding.do_transform(dataframe)
        
        '''Split the dataset'''
        train_loader, test_loader = self.split_dataset(dataset = encoded_dataset)
        
        return train_loader, test_loader
        
    
    def split_dataset(self, dataset = None):
        split_data = TrainTestSplit(dataset = dataset)
        TRAIN_LOADER, TEST_LOADER = split_data.train_test_split(dataset = dataset)
        # self.call_NN()
        # print("check - ", next(iter(TRAIN_LOADER)))
        return TRAIN_LOADER, TEST_LOADER
    
    def train_start(self, TRAIN_LOADER, TEST_LOADER):
        model = self.call_NN()
        trained_model = training_NN(model = model, TRAIN_LOADER = TRAIN_LOADER, TEST_LOADER = TEST_LOADER)
        return trained_model
    
    def evaluation_model(self, model = None, train_data = None, test_data = None):
        train_evaluation(model = model, TRAIN_LOADER = train_data)
        
        validation_evaluation(model = model, TEST_LOADER = test_data)
        
    def call_NN(self):
        model =  NeuralNetwork(input_features = 4, output_dimension = 3)
        print(model)
        print("_"*100,'\n')
        
        TOTAL_TRAINABLE = 0
        for layer_name, params in model.named_parameters():
            if params.requires_grad == True:
                print("Layer {} & Trainable Parameters # {} ".format(layer_name, params.numel()),'\n')
                print("-"*50)
                TOTAL_TRAINABLE = TOTAL_TRAINABLE + params.numel()
        
        print("Total trainable parameters # {} ".format(TOTAL_TRAINABLE).upper())
        
        print("_"*50,'\n')
        
        return model
    
        
        
if __name__ == "__main__":
    classifier = Classifier()
    # train_loader, test_loader = classifier.load_data(filename = 'C:/Users/atiku/OneDrive/Desktop/Iris.csv')
    # trained_model = classifier.train_start(TRAIN_LOADER = train_loader, TEST_LOADER = test_loader)
    # classifier.evaluation_model(model = trained_model, train_data = train_loader, test_data = test_loader)
     
