import numpy as np
import pandas as pd
import torch
import torch.nn as nn
import torch.functional as F
import torch.optim as optim
from torch.utils.data import DataLoader, Dataset
from sklearn.metrics import accuracy_score, precision_score, recall_score, f1_score, classification_report
import warnings

# # Ignore the specific warning about the deprecated softmax dimension choice
warnings.filterwarnings("ignore", category=UserWarning, message="Implicit dimension choice for softmax has been deprecated.")

def train_evaluation(model = None, TRAIN_LOADER = None):
    ACC = []
    PRE = []
    REC = []
    F1  = []
    TD  = []
    
    for (train_data, train_label) in TRAIN_LOADER:
        TD.append(train_data.shape[0])
        model_predicted = model(train_data)
        model_predcited = torch.argmax(model_predicted, dim = 1)

        ACC.append(accuracy_score(model_predcited, train_label))
        PRE.append(precision_score(model_predcited, train_label, average = 'macro'))
        REC.append(recall_score(model_predcited, train_label, average = 'macro'))
        F1.append(f1_score(model_predcited, train_label, average = 'macro'))

    print("*"*50, " TRAIN RESULTS: for {} data ".format(np.array(TD).sum()), "*"*50,'\n\n')

    print("ACCURACY  # {} ".format(np.array(ACC).mean(),'\n'))
    print("PRECISION # {} ".format(np.array(PRE).mean(),'\n'))
    print("RECALL    # {} ".format(np.array(REC).mean(),'\n'))
    print("F1_SCORE  # {} ".format(np.array(F1).mean(),'\n'))
    
    

def validation_evaluation(model = None, TEST_LOADER = None):
    ACC = []
    PRE = []
    REC = []
    F1  = []
    TD  = []
    for (val_data, val_label) in TEST_LOADER:
        TD.append(val_data.shape[0])
        model_predicted = model(val_data)
        model_predcited = torch.argmax(model_predicted, dim = 1)

        ACC.append(accuracy_score(model_predcited, val_label))
        PRE.append(precision_score(model_predcited, val_label, average = 'macro'))
        REC.append(recall_score(model_predcited, val_label, average = 'macro'))
        F1.append(f1_score(model_predcited, val_label, average = 'macro'))

    print("\n","*"*50, " TEST RESULTS for {} data: ".format(np.array(TD).sum()), "*"*50,'\n')

    print("ACCURACY  # {} ".format(np.array(ACC).mean(),'\n'))
    print("PRECISION # {} ".format(np.array(PRE).mean(),'\n'))
    print("RECALL    # {} ".format(np.array(REC).mean(),'\n'))
    print("F1_SCORE  # {} ".format(np.array(F1).mean(),'\n'))